==> swarmapps-1.0.1.tar.gz: swarmapps-1.0.1/hello-world/v03/PplObserverSwarm.m <==
// Hello, World! application. Copyright (C) 1996 Santa Fe Institute.
// This library is distributed without any warranty; without even the
// implied warranty of merchantability or fitness for a particular purpose.
// See file LICENSE for details and terms of copying.

// peculiar to some gcc function prototypes
#define __USE_FIXED_PROTOTYPES__
#include <math.h>
#import "PplObserverSwarm.h"
#import <simtools/ActiveGraph.h>

// implementation of sample graphical interface for models.
@implementation PplObserverSwarm

+ createBegin: aZone
{
  PplObserverSwarm *obj;
  id <ProbeMap> probeMap;
  
  obj = [super createBegin: aZone];

  // MODIFY: initialize other display options here.
  obj->displayFrequency = 1;

  probeMap = [EmptyProbeMap createBegin: aZone];
  [probeMap setProbedClass: [self class]];
  probeMap = [probeMap createEnd];

  // MODIFY: set any display parameters here.
  [probeMap addProbe: [probeLibrary getProbeForVariable: "displayFrequency"
				    inClass: [self class]]];

  [probeLibrary setProbeMap: probeMap For: [self class]];

  return obj;
}

// MODIFY: possible creation finalization here
- createEnd
{
  return [super createEnd];
}

- buildObjects
{
  id modelZone;
  int inci;   // increment variable
  int tempSizeX, tempSizeY;

  [super buildObjects];
  
  // allocate space to hold the model
  modelZone = [Zone create: [self getZone]];


  // create the modelswarm as a subzone in the modelzone
  pplModelSwarm = [PplModelSwarm create: modelZone];
  
  // create the probedisplay manager for the model swarm
  [probeDisplayManager createProbeDisplayFor: pplModelSwarm];
  // create the probe display for the observer swarm (self)
  [probeDisplayManager createProbeDisplayFor: self];

  // set the control panel state
  [controlPanel waitForControlEvent];
  // test to see if the "quit" button was hit
  if ([controlPanel getState] == ControlStateQuit)
    return self;

  // build the objects defined in PplmodelSwarm.m
  [pplModelSwarm buildObjects];

  // display the counter
  observedCounter = [pplModelSwarm getCounter];


  // creat a graph of numberPplStillTalking using the BLT
  // graph straight out of the library
  pplLeftGraph = [BLTGraph create: [self getZone]];
  [pplLeftGraph title: "Number of Ppl Still Talking"];
  [pplLeftGraph axisLabelsX: "Cycle" Y: "Number Left"];
  [pplLeftGraph setWidth: 400 Height: 250];
  [pplLeftGraph pack];
  [pplLeftGraph setScaleModeX: 1 Y: 0];

  // put the element on the graph
  numberLeft = [pplLeftGraph createElement];
  [[[numberLeft setLabel: "Number Left"] setColor: "green"]
    setWidth: 3];
  pplLeftGrapher = [ActiveGraph createBegin: [self getZone]];
  [pplLeftGrapher setElement: numberLeft];
  [pplLeftGrapher setDataFeed: pplModelSwarm];
  [pplLeftGrapher setProbedSelector: M(getNumberPplStillTalking)];
  pplLeftGrapher = [pplLeftGrapher createEnd];

  // create a graph of the average number of people talked to over
  //    the whole population
  avgFriendGraph = [EZGraph createBegin: [self getZone]];
  [avgFriendGraph setTitle: "Avg. # of Friends"];
  [avgFriendGraph setAxisLabelsX: "Cycle" Y: "Avg. Friends"];
  avgFriendGraph = [avgFriendGraph createEnd];

  [avgFriendGraph createAverageSequence: "avgNumFriends"
		  withFeedFrom: [pplModelSwarm getPplList]
		  andSelector: M(getNumFriends)];

  // Now draw the space

  colormap = [XColormap create: [self getZone]];
  // set up the colormap
  for (inci = 0; inci < 125; inci++)
    {
      [colormap setColor: inci ToRed: sin((inci/125.0)*3.14159 ) 
                Green: cos ((inci/125.0)*(3.14159/2.0) - (3.14159/2.0))
                Blue: sin ((inci/125.0)*(3.14159/2.0) + (3.14159/2.0))];
    }
  // Color 125 is set to white, used for people who've left the party
  [colormap setColor: 125 ToName: "white"];

  // create the 2d window
  worldRaster = [ZoomRaster create: [self getZone]];
  [worldRaster setColormap: colormap];

  // set up the zoom factor based on how big the world is
  tempSizeX = [[pplModelSwarm getWorld] getSizeX];
  tempSizeY = [[pplModelSwarm getWorld] getSizeY];
  [worldRaster setZoomFactor: 320/tempSizeX];
  [worldRaster setWidth: tempSizeX Height: tempSizeY];

  [worldRaster setWindowTitle: "Room 1"];
  [worldRaster pack];

  // And also create an Object2dDisplay: this object draws ppl on
  // the worldRaster widget for us, and also receives probes.
  pplDisplay = [Object2dDisplay createBegin: [self getZone]];
  [pplDisplay setDisplayWidget: worldRaster];
  [pplDisplay setDiscrete2dToDisplay: [pplModelSwarm getWorld]];
  [pplDisplay setObjectCollection: [pplModelSwarm getPplList]];
  [pplDisplay setDisplayMessage: M(drawSelfOn:)];   // draw method
  pplDisplay = [pplDisplay createEnd];

  // Also, tell the world raster to send mouse clicks to the pplDisplay
  // this allows the user to right-click on the display to probe the bugs.
  [worldRaster setButton: ButtonRight Client: pplDisplay Message: M(makeProbeAtX:Y:)];


  return self;
}  

- buildActions
{
  // build the actions for the observer's schedule
  [super buildActions];
  
  // build the actions for the model's schedule
  [pplModelSwarm buildActions];

  // create the display actions
  displayActions = [ActionGroup create: [self getZone]];
  // update the counter
  //  [displayActions createActionTo: observedCounter     message: M(print)];
  // update the room display
  [displayActions createActionTo: worldRaster         message: M(erase)];
  [displayActions createActionTo: pplDisplay      message: M(display)];
  [displayActions createActionTo: worldRaster         message: M(drawSelf)];
  // update the graph
  [displayActions createActionTo: pplLeftGrapher        message: M(step)];
  // update the average number of ppl talked to
  [displayActions createActionTo: avgFriendGraph message: M(step)];
  // check to see if everybody's done talking
  [displayActions createActionTo: self                message: M(checkToStop)];
  // update the probes
  [displayActions createActionTo: probeDisplayManager message: M(update)];
  // check the controlpanel for clicks
  [displayActions createActionTo: actionCache        message: M(doTkEvents)];

  // then actually schedule the display actions
  displaySchedule = [Schedule createBegin: [self getZone]];
  [displaySchedule setRepeatInterval: displayFrequency];
  displaySchedule = [displaySchedule createEnd];
  [displaySchedule at: 0 createAction: displayActions];

  return self;
}  

- activateIn: swarmContext
{
  // put self (observer swarm) on the schedule of the GUI swarm
  [super activateIn: swarmContext];

  // start the activity for the model swarm
  [pplModelSwarm activateIn: self];
  // start the display schedule
  [displaySchedule activateIn: self];
  
  return [self getSwarmActivity];
}

// this method queries the pplModelSwarm to find out if everybody
// has sent the ImFinished message to the modelswarm.  If so,
// then stop the sim because nobody's talking.
- checkToStop
{
  if ([pplModelSwarm getNumberPplStillTalking] <= 0) {
    (void) printf("Party's over!  You don't have to go home;"
		  "  but you can't stay here.\n");
    [controlPanel setStateStopped];
  }
  return self;
}

@end

==> swarmapps-1.0.2.tar.gz: swarmapps-1.0.2/hello-world/v03/PplObserverSwarm.m <==
// Hello, World! application. Copyright (C) 1996-1998 Santa Fe Institute.
// This library is distributed without any warranty; without even the
// implied warranty of merchantability or fitness for a particular purpose.
// See file LICENSE for details and terms of copying.

// peculiar to some gcc function prototypes
#define __USE_FIXED_PROTOTYPES__
#include <math.h>
#import "PplObserverSwarm.h"
#import <simtools/ActiveGraph.h>

// implementation of sample graphical interface for models.
@implementation PplObserverSwarm

+ createBegin: aZone
{
  PplObserverSwarm *obj;
  id <ProbeMap> probeMap;
  
  obj = [super createBegin: aZone];

  // MODIFY: initialize other display options here.
  obj->displayFrequency = 1;

  probeMap = [EmptyProbeMap createBegin: aZone];
  [probeMap setProbedClass: [self class]];
  probeMap = [probeMap createEnd];

  // MODIFY: set any display parameters here.
  [probeMap addProbe: [probeLibrary getProbeForVariable: "displayFrequency"
				    inClass: [self class]]];

  [probeLibrary setProbeMap: probeMap For: [self class]];

  return obj;
}

// MODIFY: possible creation finalization here
- createEnd
{
  return [super createEnd];
}

- buildObjects
{
  id modelZone;
  int inci;   // increment variable
  int tempSizeX, tempSizeY;

  [super buildObjects];
  
  // allocate space to hold the model
  modelZone = [Zone create: [self getZone]];


  // create the modelswarm as a subzone in the modelzone
  pplModelSwarm = [PplModelSwarm create: modelZone];
  
  // create the probedisplay manager for the model swarm
  CREATE_PROBE_DISPLAY (pplModelSwarm);
  // create the probe display for the observer swarm (self)
  CREATE_PROBE_DISPLAY (self);

  // set the control panel state
  [controlPanel waitForControlEvent];
  // test to see if the "quit" button was hit
  if ([controlPanel getState] == ControlStateQuit)
    return self;

  // build the objects defined in PplmodelSwarm.m
  [pplModelSwarm buildObjects];

  // display the counter
  observedCounter = [pplModelSwarm getCounter];


  // creat a graph of numberPplStillTalking using the BLT
  // graph straight out of the library
  pplLeftGraph = [BLTGraph create: [self getZone]];
  [pplLeftGraph title: "Number of Ppl Still Talking"];
  [pplLeftGraph axisLabelsX: "Cycle" Y: "Number Left"];
  [pplLeftGraph setWidth: 400 Height: 250];
  [pplLeftGraph pack];
  [pplLeftGraph setScaleModeX: 1 Y: 0];

  // put the element on the graph
  numberLeft = [pplLeftGraph createElement];
  [[[numberLeft setLabel: "Number Left"] setColor: "green"]
    setWidth: 3];
  pplLeftGrapher = [ActiveGraph createBegin: [self getZone]];
  [pplLeftGrapher setElement: numberLeft];
  [pplLeftGrapher setDataFeed: pplModelSwarm];
  [pplLeftGrapher setProbedSelector: M(getNumberPplStillTalking)];
  pplLeftGrapher = [pplLeftGrapher createEnd];

  // create a graph of the average number of people talked to over
  //    the whole population
  avgFriendGraph = [EZGraph createBegin: [self getZone]];
  [avgFriendGraph setTitle: "Avg. # of Friends"];
  [avgFriendGraph setAxisLabelsX: "Cycle" Y: "Avg. Friends"];
  avgFriendGraph = [avgFriendGraph createEnd];

  [avgFriendGraph createAverageSequence: "avgNumFriends"
		  withFeedFrom: [pplModelSwarm getPplList]
		  andSelector: M(getNumFriends)];

  // Now draw the space

  colormap = [XColormap create: [self getZone]];
  // set up the colormap
  for (inci = 0; inci < 125; inci++)
    {
      [colormap setColor: inci ToRed: sin((inci/125.0)*3.14159 ) 
                Green: cos ((inci/125.0)*(3.14159/2.0) - (3.14159/2.0))
                Blue: sin ((inci/125.0)*(3.14159/2.0) + (3.14159/2.0))];
    }
  // Color 125 is set to white, used for people who've left the party
  [colormap setColor: 125 ToName: "white"];

  // create the 2d window
  worldRaster = [ZoomRaster create: [self getZone]];
  [worldRaster setColormap: colormap];

  // set up the zoom factor based on how big the world is
  tempSizeX = [[pplModelSwarm getWorld] getSizeX];
  tempSizeY = [[pplModelSwarm getWorld] getSizeY];
  [worldRaster setZoomFactor: 320/tempSizeX];
  [worldRaster setWidth: tempSizeX Height: tempSizeY];

  [worldRaster setWindowTitle: "Room 1"];
  [worldRaster pack];

  // And also create an Object2dDisplay: this object draws ppl on
  // the worldRaster widget for us, and also receives probes.
  pplDisplay = [Object2dDisplay createBegin: [self getZone]];
  [pplDisplay setDisplayWidget: worldRaster];
  [pplDisplay setDiscrete2dToDisplay: [pplModelSwarm getWorld]];
  [pplDisplay setObjectCollection: [pplModelSwarm getPplList]];
  [pplDisplay setDisplayMessage: M(drawSelfOn:)];   // draw method
  pplDisplay = [pplDisplay createEnd];

  // Also, tell the world raster to send mouse clicks to the pplDisplay
  // this allows the user to right-click on the display to probe the bugs.
  [worldRaster setButton: ButtonRight Client: pplDisplay Message: M(makeProbeAtX:Y:)];


  return self;
}  

- buildActions
{
  // build the actions for the observer's schedule
  [super buildActions];
  
  // build the actions for the model's schedule
  [pplModelSwarm buildActions];

  // create the display actions
  displayActions = [ActionGroup create: [self getZone]];
  // update the counter
  //  [displayActions createActionTo: observedCounter     message: M(print)];
  // update the room display
  [displayActions createActionTo: worldRaster         message: M(erase)];
  [displayActions createActionTo: pplDisplay      message: M(display)];
  [displayActions createActionTo: worldRaster         message: M(drawSelf)];
  // update the graph
  [displayActions createActionTo: pplLeftGrapher        message: M(step)];
  // update the average number of ppl talked to
  [displayActions createActionTo: avgFriendGraph message: M(step)];
  // check to see if everybody's done talking
  [displayActions createActionTo: self                message: M(checkToStop)];
  // update the probes
  [displayActions createActionTo: probeDisplayManager message: M(update)];
  // check the controlpanel for clicks
  [displayActions createActionTo: actionCache        message: M(doTkEvents)];

  // then actually schedule the display actions
  displaySchedule = [Schedule createBegin: [self getZone]];
  [displaySchedule setRepeatInterval: displayFrequency];
  displaySchedule = [displaySchedule createEnd];
  [displaySchedule at: 0 createAction: displayActions];

  return self;
}  

- activateIn: swarmContext
{
  // put self (observer swarm) on the schedule of the GUI swarm
  [super activateIn: swarmContext];

  // start the activity for the model swarm
  [pplModelSwarm activateIn: self];
  // start the display schedule
  [displaySchedule activateIn: self];
  
  return [self getSwarmActivity];
}

// this method queries the pplModelSwarm to find out if everybody
// has sent the ImFinished message to the modelswarm.  If so,
// then stop the sim because nobody's talking.
- checkToStop
{
  if ([pplModelSwarm getNumberPplStillTalking] <= 0) {
    (void) printf("Party's over!  You don't have to go home;"
		  "  but you can't stay here.\n");
    [controlPanel setStateStopped];
  }
  return self;
}

@end
